package org.bitbucket.bradleysmithllc.etlunit.feature.execute;

import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.execute.java_executor.ExecuteHandler;
import org.bitbucket.bradleysmithllc.etlunit.feature.execute.java_executor.ExecuteRequest;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.List;
import java.util.Map;

public class JavaExecutor extends NullClassListener implements ExecuteHandler {
	@Override
	public action_code execute(ExecuteRequest request, ETLTestOperation testOperation, ETLTestValueObject valueObject, VariableContext variableContext, ExecutionContext executionContext)
			throws TestAssertionFailure, TestExecutionError, TestWarning
	{
		String className = request.getJavaClass();

		try
		{
			Class cl = Class.forName(className);

			Method mth = cl.getDeclaredMethod("main", new Class [] {String[].class});

			int mods = mth.getModifiers();

			if (!Modifier.isPublic(mods))
			{
				throw new TestExecutionError("Java class has main method, but it isn't public: " + mth, ExecuteConstants.ERR_JAVA_MAIN_NOT_PUBLIC);
			}

			if (!Modifier.isStatic(mods))
			{
				throw new TestExecutionError("Java class has public main method, but it isn't static: " + mth, ExecuteConstants.ERR_JAVA_MAIN_NOT_STATIC);
			}

			// set a debug token in the system properties to identify this test
			System.setProperty("etlunit.active", "true");
			System.setProperty("etlunit.test.class", testOperation.getTestClass().getQualifiedName());
			System.setProperty("etlunit.test.method", testOperation.getTestMethod().getName());
			System.setProperty("etlunit.test.method", testOperation.getTestMethod().getName());
			System.setProperty("etlunit.test.operation", testOperation.getOperationName());
			System.setProperty("etlunit.test.operation.ordinal", String.valueOf(testOperation.getOrdinal()));
			System.setProperty("etlunit.test.operation.qualified", testOperation.getQualifiedName());

			List arguments = request.getArguments();

			String [] args;

			if (request.hasArguments())
			{
				args = new String[arguments.size()];

				for (int i = 0; i < args.length; i++)
				{
					args[i] = String.valueOf(arguments.get(i));
				}
			}
			else
			{
				args = new String[0];
			}

			if (request.hasProperties())
			{
				System.getProperties().putAll(request.getProperties());
			}

			mth.invoke(null, new Object[]{args});
		}
		catch (ClassNotFoundException e)
		{
			throw new TestExecutionError("Java class not found: " + className, ExecuteConstants.ERR_JAVA_CLASS_NOT_FOUND, e);
		}
		catch (NoSuchMethodException e)
		{
			throw new TestExecutionError("Java class found, but it does not have a main method: " + className, ExecuteConstants.ERR_JAVA_MAIN_METHOD_NOT_FOUND, e);
		}
		catch (InvocationTargetException e)
		{
			throw new TestExecutionError("Java execution failure: " + className, ExecuteConstants.ERR_JAVA_EXECUTION_ERROR, e);
		}
		catch (IllegalAccessException e)
		{
			throw new TestExecutionError("Java class has main method, but it isn't accessible: " + className, ExecuteConstants.ERR_JAVA_MAIN_NOT_PUBLIC, e);
		}

		return action_code.handled;
	}
}
